#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>

#define F_CPU	3690000
#define UART_BAUD_RATE      9600      /* 9600 baud */


#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)

/*
*/


typedef unsigned char  u08;
typedef          char  s08;
typedef unsigned short u16;
typedef          short s16;

static volatile u08 *uart_data_ptr;
static volatile u08 uart_counter;

static volatile u16 adcData;
static volatile u08 adcDataL;
static volatile u08 adcDataH;

static char msg[6] = "S-IN";

static char newline[3] = "\r\n";


uint8_t g_second = 0;
uint8_t g_minute = 0;
uint8_t g_hour   = 0;
uint8_t g_month  = 0;
uint8_t g_day    = 0;
uint8_t g_year   = 0;

static volatile uint8_t g_commandStarted = 0;
volatile uint8_t count = 0;
volatile uint8_t printed = 0;
volatile char g_command[40];
volatile char out[10];

static void delay(uint16_t us);

volatile uint8_t conversion = 0;

typedef struct
{
	uint8_t second;
	uint8_t minute;
	uint8_t hours;
	uint8_t month;
	uint8_t day;
	uint8_t year;

	uint8_t enabled;
	uint8_t pind;	
	uint8_t action;
} event_t;

volatile event_t events[3];
volatile uint8_t maxEvents = 3;
volatile uint8_t numberEvents = 0;

SIGNAL(SIG_ADC)
{
    adcDataL = ADCL;
    adcDataH = ADCH;
    
    adcData = 0;
    adcData = adcData | adcDataH;
    adcData = adcData << 8;
    adcData = adcData | adcDataL;
    
    //adcData = ((8<<adcDataH)|adcDataL); 
}


SIGNAL(SIG_UART_TRANS)      
/* signal handler for uart txd ready interrupt */
{
    uart_data_ptr++;

    if (--uart_counter)
        outp(*uart_data_ptr, UDR);       /* write byte to data buffer */
}

void uart_send(u08 *buf, u08 size)
/* send buffer <buf> to uart */
{   
    if (!uart_counter) 
    {
        /* write first byte to data buffer */
        uart_data_ptr  = buf;
        uart_counter   = size;
        outp(*buf, UDR);
    }
}


void sendLineUART(char* text)
{
    uart_send(text, 4);
    while (uart_counter != 0) { }
}

void writePortB()
{
    int i = 0;
        
    for (i=1; i<5; i++)
    {
        if (g_command[i] == '1')
        {
            cbi(PORTB, (i-1));
        }
        else
        {
            sbi(PORTB, (i-1));
        }
    }  
}

void superdelay()
{
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);
    delay(10000);

}

volatile char steve[10];

void readADC(uint8_t channel)
{
    if (channel == 0)
    {
        ADMUX = 0;
    }
    else if (channel == 1)
    {
        ADMUX = 1;
    }
    else if (channel == 2)
    {
        ADMUX = 2;
    }
    else if (channel == 3)
    {
        ADMUX = 3;
    }
        
    
    ADCSR |= (1<<ADSC);
    delay(10000);

    superdelay();
    
    out[0] = 'S';
    uart_send(out, 1);
    while (uart_counter != 0) { }   

    out[0] = 'A';
    uart_send(out, 1);
    while (uart_counter != 0) { }   

    itoa(adcData, steve, 10);
    uart_send(steve, 4);
    while (uart_counter != 0) { }       
    
        
    out[0] = 'Z';
    uart_send(out, 1);
    while (uart_counter != 0) { }       
}

void readPortD()
{
    int i = 0;

    superdelay();
    
    out[0] = 'S';
    uart_send(out, 1);
    while (uart_counter != 0) { }

    out[0] = 'R';
    uart_send(out, 1);
    while (uart_counter != 0) { }
    
    for (i=4; i<8; i++)
    {
        if (bit_is_set(PINC, i))
        {
            out[0] = '0';    
            uart_send(out, 1);
            while (uart_counter != 0) { }

        }
        else
        {
            out[0] = '1';
            uart_send(out, 1);
            while (uart_counter != 0) { }

        }
    }

    out[0] = 'Z';
    uart_send(out, 1);
    while (uart_counter != 0) { } 


    //}
}

void processCommand()
{
    uint8_t channel = 0;
    char channel_s[2];
    
    channel_s[1] = '\0';
    
	if (g_command[0] == 'T')			
	{
	}
	else if (g_command[0] == 'E')		
	{
	}
	else if (g_command[0] == 'A')		
	{
            channel_s[0] = g_command[1];
            channel = atoi(channel_s);
            readADC(channel);
	}
	else if (g_command[0] == 'R')		
	{
            readPortD();
	}
	else if (g_command[0] == 'B')		
	{
            writePortB();
	}
	else if (g_command[0] == 'D')		
	{
	}
	
}

SIGNAL(SIG_UART_RECV)      
/* signal handler for receive complete interrupt */
{
        register unsigned char inByte;

        inByte = inp(UDR);        /* read byte for UART data buffer */
	
	if (inByte == 'Z')
	{		
		printed = 1;
		g_commandStarted = 0;
		count = 0;
	}

	if (!g_commandStarted) 
	{
		if (inByte == 'S') 
		{	
			g_commandStarted = 1;
			memset(g_command, ' ', 39);	
		}

	}
	else
	{
            g_command[count] = inByte;
            count++;
	
            if (count > 39)
                count = 39;
    }
	
   // outp(~inByte, PORTB);     /* output received byte to PortB (LEDs) */
}




static void delay(uint16_t us) 
{
    while(us)
    	us--;
}



void uart_init(void)
/* initialize uart */
{
    /* enable RxD/TxD and ints */
    outp((1<<RXCIE)|(1<<TXCIE)|(1<<RXEN)|(1<<TXEN), UCSRB);       
	
    /* set baud rate */
    outp((u08)UART_BAUD_SELECT, UBRR);          
}

void adc_init(void)
{
    ADCSR |= ( (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0) );
    //ADCSR |= (1<<ADFR); //Enable free-running mode
    ADMUX = 0; // ((0<<ADLAR)); 
    ADCSR |= (1<<ADIE); //Enable ADC conversion complete interrupt
    ADCSR |= (1<<ADEN); //Enable the ADC
    //ADCSR |= (1<<ADSC); //Start first conversion in Free-running mode
    //Now the conversions are running
    //Channels are changed in interrupt service routine
}


int main(void)
{
    uint8_t inByte;
    
    outp(0xff ,DDRB);      /* PortB output */
    outp(0x00, PORTB);     /* switch LEDs on */

    //inByte = inb(DDRC);
    //inByte = inByte ^ 0xf0;
    outp(0x00, DDRC);       /* PortC input only */
    
    memset(g_command, ' ', 39);
	
    adc_init();
    
    uart_init();
    
    sei();                 /* enable interrupts */

    while(1)              /* loop forever */
    {
        if (printed)
        {
            /*
	    uart_send(g_command, 39);
	    while (uart_counter != 0) { }
			
	    uart_send(newline, 3);
    	    while (uart_counter != 0) { } 
            
            */
            processCommand();
	    printed = 0;
	}
		
        
	/*
   		uart_send(msg3, 6);
		while (uart_counter != 0) { }
        
        itoa(adcData, steve, 10);   
        uart_send(steve, 10);     
   		while (uart_counter != 0) { }

        uart_send(newline, 3);
    	while (uart_counter != 0) { }  
	*/
  	//	uart_send(msg3, 6);
	//	while (uart_counter != 0) { }
        
	//	uart_send(newline, 3);
    //	while (uart_counter != 0) { } 
    }
}
