/*
    PalmLCD - Accepts draw commands via serial port and acts as a slave

    Steve Manley (smanley@nyx.net)

    Feb 25th 2000 - Added Basic Functionality - ALL YOU BASE BELONG TO US	


    // Todos
    //
    // Get integer from checkboxes
    // Write integer to checkboxes
    // Send a ADC channel request
    // Read ADC value and write to screen
    // Read integer (port) value
    // Write integer port value
    //
    // Send S to start command, Z to send/reset/echo
    // 
    // SADC0Z - read adc
    // SWR[byte1][byte2]Z - adc value (merge)
    // SPB[byte]Z - write port b
    // SPD[byte]Z - recieve port D setting
*/

#include <PalmOS.h>
#include <PalmCompatibility.h>				
#include <System/SerialMgrOld.h>

#include "palmlcdrsc.h"

// Application globals
UInt16   gSerialLib;
int      gSerialPortOpen;
short    gScreenBuffer[160];
char     gSerialBuffer[400];

char     gCommand[10];
char     gText[80];

char     gSerialCommand[10];

int      gCommandStarted;
int      gXpoint;
int      gYpoint;

ULong       gLastSuccessfulReception = 0;       
ULong       gLastTimeDisplay = 0;
ULong       gFormOpened = false;
ULong       gUpdateTimer;

unsigned char g_portB = 0;
unsigned char g_portD = 0;

int adcChannel = 0;
int adcValue = 0;

int g_adcType = 0;

void drawScreen()
{
}


void setPoint(int x, int y, int value)
{
    int i,j;
    for (i=0; i<160; i++)
    {
            WinDrawLine(i,i,i,i);
    }
}

/*
    Processes the command from the serial port
*/
void handleCommandRequest()
{
    int bufferLength;
    int i;
    Err err;

    bufferLength = StrLen(gSerialBuffer);
    if ((!gCommandStarted) && (bufferLength > 10))
    {
        gSerialBuffer[0] = '\0';
        return;
    }

    if (gSerialBuffer[bufferLength - 1] == '*')
    {
        gSerialBuffer[0] = '\0';
        SerSend(gSerialLib, "reset", 5, &err);
        return;
    }

    if (!gCommandStarted)
    {
        if (StrCompare(gSerialBuffer, "POINT") == 0)
        {
            SerSend(gSerialLib, "point", 5, &err);
            StrCopy(gCommand, "POINT");
            gCommandStarted = 1;
            gSerialBuffer[0] = '\0';
        }

        if (StrCompare(gSerialBuffer, "ERASE") == 0)
        {
            SerSend(gSerialLib, "erase", 5, &err);
            StrCopy(gCommand, "ERASE");
            gCommandStarted = 1;
            gSerialBuffer[0] = '\0';
        }
    
        if (StrCompare(gSerialBuffer, "TEXT") == 0)
        {
            SerSend(gSerialLib, "text", 4, &err);
            StrCopy(gCommand, "TEXT");
            gCommandStarted = 1;
            gSerialBuffer[0] = '\0';
        }

        gXpoint = -1;
        gYpoint = -1;
    }
    else
    {
        bufferLength = StrLen(gSerialBuffer);

        for (i=0; i<bufferLength; i++)
        {
            if ((gSerialBuffer[i] == '|') && (StrCompare(gCommand, "TEXT") != 0))
            {   
                gSerialBuffer[i] = '\0';
        
                if (gXpoint == -1)
                    gXpoint = StrAToI(gSerialBuffer);
                
                else if (gYpoint == -1)  
                    gYpoint = StrAToI(gSerialBuffer);
                
                gSerialBuffer[0] = '\0';
            }
            else
            {
                // Text buffer
                if (gSerialBuffer[i] == '|')
                {
                    gSerialBuffer[i] = '\0';

                    if (gXpoint == -1)
                        gXpoint = StrAToI(gSerialBuffer);
                
                    else if (gYpoint == -1)  
                        gYpoint = StrAToI(gSerialBuffer);
 
                    else 
                        StrCopy(gText, gSerialBuffer);

                    gSerialBuffer[0] = '\0';
                }
            }
        }
    }

    if ((gXpoint != -1) && (gYpoint != -1))
    {
        if (StrCompare(gCommand, "POINT") == 0)
        {
            SerSend(gSerialLib, "Drawing!", 8, &err);
            WinDrawLine(gXpoint,gYpoint,gXpoint,gYpoint);
            gXpoint = gYpoint = -1;
            gCommandStarted = 0;
        }
        else if (StrCompare(gCommand, "ERASE") == 0)
        {
            SerSend(gSerialLib, "Drawing!", 8, &err);
            WinInvertLine(gXpoint,gYpoint,gXpoint,gYpoint);
            gXpoint = gYpoint = -1;
                 SerSend(gSerialLib, "Read", 4, &err);     gCommandStarted = 0;
        }
        else if ((StrCompare(gCommand, "TEXT") == 0) && (gText[0] != '\0') )
        {
            SerSend(gSerialLib, "Drawing!", 8, &err);
            WinDrawChars (gText, StrLen(gText), gXpoint, gYpoint);
            gXpoint = gYpoint = -1;
            gText[0] = '\0';
            gCommandStarted = 0;
        }
    }
}


void handleSerial()
{

}

/*
    Handle recieving commands from the serial port

*/

int gCommandStarted = 0;
int count = 0;

void handleSerialReceive(void)
{
    int     i;
    char    data[80];
    char    temp[6];
    Err     err;
    ULong   numberBytesPending;
    ULong   numberBytesRead;
    Boolean handled;

    FormPtr form;
    TablePtr table;
    FieldPtr field;
    MemHandle textHandle;
    char* txt;
    
    char command;
    int index = 0;
    
    data[0] = '\0';
    
    SerClearErr(gSerialLib);
    SerReceiveFlush(gSerialLib, 1);
    
    form = FrmGetActiveForm();
     
    //SerSend(gSerialLib, "Handled:", 8, &err);
    
    do
    {
        SerReceiveWait(gSerialLib, sizeof(data), sysTicksPerSecond/50);
        SerReceiveCheck(gSerialLib, &numberBytesPending);

        if (numberBytesPending > 0) 
        {
            //numberBytesRead = SerReceive(gSerialLib, data, numberBytesPending, 0, &err);
            SerReceive(gSerialLib, data, numberBytesPending, 0, &err);
            //SerReceive(gSerialLib, data, 1, 0, &err);

            //SerSend(gSerialLib, data,  numberBytesPending, &err);
            //SerSend(gSerialLib, "-Read\n\r", 7, &err);
             

            while (numberBytesPending)
            {
            command = data[index];
            numberBytesPending--;
            index++;
            
    
            if (gCommandStarted)
            {
                gSerialCommand[count] = command;
                count++;
                gSerialCommand[count] = '\0';
            }
            
            if (command == 'S')
            {
                gCommandStarted = 1;
                count = 0;
            }

            if ( (command == 'Z') && (gCommandStarted) )
            {
                gCommandStarted = 0;
                count = 0;
                
                //SerSend(gSerialLib, gSerialCommand,  StrLen(gSerialCommand), &err);

                if (gSerialCommand[0] == 'R')
                {
                    //SerSend(gSerialLib, "OK\r\n", 4, &err);

                    table = FrmGetObjectPtr(form, FrmGetObjectIndex (form, portDTable));
             
                    for (i = 0; i < 4; i++)
                    {
                        if (gSerialCommand[(i+1)] == '0')
                        {
                            TblSetItemInt(table, i, 1, 0);
                        }
                        else
                        {
                            TblSetItemInt(table, i, 1, 1);
                        }
                    }      

                    TblMarkTableInvalid(table);
                    TblRedrawTable(table);
                    TblDrawTable(table);

                    FrmDrawForm(FrmGetActiveForm());
                } 

                if (gSerialCommand[0] == 'A')
                {   
                    for (i=0; i<4; i++)
                    {
                        temp[i] = gSerialCommand[(i+1)];
                    }

                    SerSend(gSerialLib, temp, 4, &err);

                    field = FrmGetObjectPtr(form, FrmGetObjectIndex(form, adcLabelValue));
                    
                    textHandle = MemHandleNew (8);
                    txt = MemHandleLock(textHandle);
                    StrNCopy(txt, temp, 4);
                    txt[4] = '\0';
                    FldSetTextHandle(field, textHandle);
                    MemHandleUnlock(textHandle);
                    
                    FldDrawField(field);
                }
            }
                      
            //StrCat(gSerialBuffer, data);
            
            //handleCommandRequest();

            SerReceiveFlush(gSerialLib, 1);
            SerClearErr(gSerialLib);

            
            }
            index = 0;

        }
    }
    while (!EvtSysEventAvail(false)); // Avoid blocking indefinately

}

static Err StartApplication(void)
{
    Err err;
    static SerSettingsType serialSettings;
    int i;

    FrmGotoForm(PalmLCDForm);

    if (SysLibFind ("Serial Library", &gSerialLib)) 
    {
        FrmAlert (SerialAlert);
    }

    err = SerOpen(gSerialLib, 0, 9600);

    if (err != 0) 
    {
        if (err == serErrAlreadyOpen) 
        {
            SerClose(gSerialLib);
            ErrNonFatalDisplay("Serial library is busy");
        } 
        else
        {  
            ErrNonFatalDisplay("Can't open serial library");
        }
 
        gSerialPortOpen = false;
 
        return true;
    }
    
    gSerialPortOpen = true;
    gCommandStarted = false;
    gSerialBuffer[0] = '\0';
    gText[0] = '\0';
    gXpoint = -1;
    gYpoint = -1;

    // Turn off hardware handshaking here
    serialSettings.baudRate = 9600;
    serialSettings.flags = 0;
    serialSettings.ctsTimeout = 0;      

    serialSettings.flags = serSettingsFlagStopBits1 | serSettingsFlagBitsPerChar8;

    err = SerSetSettings(gSerialLib, &serialSettings);

    if (err)
    {
        SerClearErr(gSerialLib);
        ErrDisplay("Error setting serial port");
    }

    //SerSend(gSerialLib, "Start", 5, &err);

    /*

    err = SerSetReceiveBuffer(gSerialLib, gSerialBuffer, sizeof(gSerialBuffer));

    if (err != 0)
    {
        ErrNonFatalDisplay("Can't set serial buffer!");
    }
    */

    /* Set the screen to completely off */
    for (i=0; i<160; i++)
    {
        gScreenBuffer[i] = 0;
    }

   // ErrNonFatalDisplay("Done init..");

    return 0;
}


static void StopApplication(void)
{
    if (gSerialPortOpen) 
    {
        // SerSetReceiveBuffer(gSerialLib, NULL, 0);
        SerClose(gSerialLib);
    }
}

void customDrawPinB(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds)
{
    char *txt;
    char buffer[5];
    MemHandle mem_handle;
    FontID  old_font;

    mem_handle = MemHandleNew (16 + 1);
    txt = MemHandleLock (mem_handle);

    StrIToA(buffer, row);
    StrCopy(txt, "PINB");   
    txt[4] = buffer[0];
    txt[5] = '\0';

    old_font = FntSetFont (0);
    WinDrawChars (txt, StrLen(txt), bounds->topLeft.x, bounds->topLeft.y);
    FntSetFont (old_font);

    MemHandleUnlock (mem_handle);      
}

void customDrawPinD(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds)
{
    char *txt;
    char buffer[5];
    MemHandle mem_handle;
    FontID  old_font;

    mem_handle = MemHandleNew (16 + 1);
    txt = MemHandleLock (mem_handle);

    StrIToA(buffer, (4 + row) );
    StrCopy(txt, "PINC");    
    txt[4] = buffer[0];
    txt[5] = '\0';

    old_font = FntSetFont (0);
    WinDrawChars (txt, StrLen(txt), bounds->topLeft.x, bounds->topLeft.y);
    FntSetFont (old_font);

    MemHandleUnlock (mem_handle);      
}


static Boolean MyFormHandleEvent(EventPtr event)
{
    int i;
    char temp[6];
    
    Boolean handled;
    Err err;

    ListPtr adcList;
    ControlPtr adcCtl;
    FormPtr form;
    
    TablePtr table;
    
    FieldPtr field;
    MemHandle textHandle;
    char* txt;
    
    handled = false;

    form = FrmGetActiveForm();
    
    switch (event->eType)
    {
        case nilEvent:

            handled = true;
            handleSerialReceive();

        break;
        
        case ctlSelectEvent:  // A control button was pressed and released.
   	    
            switch (event->data.ctlEnter.controlID)
            {
                case adcButton:

                    adcList = FrmGetObjectPtr(form, FrmGetObjectIndex(form, adcChannelList));
                    adcChannel = (int) (LstGetSelection(adcList) );               

                    SerSend(gSerialLib, "S", 1, &err);
                    SerSend(gSerialLib, "A", 1, &err);

                    
                    StrIToA(temp, adcChannel);
                    SerSend(gSerialLib, temp,  StrLen(temp), &err);                
                   
                    SerSend(gSerialLib, "Z", 1, &err);
                                        
                    handled = true;
                    
                break;

                case portTableWrite:

                    SerSend(gSerialLib, "S", 1, &err);
                    SerSend(gSerialLib, "B", 1, &err);

                    table = FrmGetObjectPtr(form, FrmGetObjectIndex (form, portBTable));
             
                    for (i = 0; i < 4; i++)
                    {
                        if(TblGetItemInt(table, i, 1))
                        {
                            SerSend(gSerialLib, "1", 1, &err);
                        }
                        else
                        {
                            SerSend(gSerialLib, "0", 1, &err);
                        }                               
                    }

                    SerSend(gSerialLib, "Z", 1, &err);
                    //SerSend(gSerialLib, "\n\r",  2, &err);  
                    
                    handled = true;
                    
                break;

                case portTableRead:

                    SerSend(gSerialLib, "S", 1, &err);
                    SerSend(gSerialLib, "R", 1, &err);
                    SerSend(gSerialLib, "Z", 1, &err);
                    
                    handled = true;

                break;                
            }
        
	
        break;

  	case frmOpenEvent:	

            field = FrmGetObjectPtr(form, FrmGetObjectIndex(form, adcLabelValue));

            textHandle = MemHandleNew (8);
            txt = MemHandleLock(textHandle);
            
            StrCopy(txt, "0000");
            
            FldSetTextHandle(field, textHandle);
            MemHandleUnlock(textHandle);
            FldDrawField(field);      
        
            adcList = FrmGetObjectPtr(form, FrmGetObjectIndex(form, adcChannelList));
            adcCtl = FrmGetObjectPtr(form, FrmGetObjectIndex(form, adcChannelTrigger));
            
            LstSetSelection (adcList, g_adcType);
            LstSetTopItem   (adcList, g_adcType);

            CtlSetLabel(adcCtl, LstGetSelectionText(adcList, 0 ));

            // Draw the pin output tables - table B

            table = FrmGetObjectPtr(form, FrmGetObjectIndex (form, portBTable));
            
            TblEraseTable(table);
            TblSetColumnUsable (table, 0, true);
            TblSetColumnUsable (table, 1, true);    
            
            TblSetCustomDrawProcedure (table, 0, customDrawPinB);
   
            for (i = 0; i < 4; i++)
            {
                TblSetRowID(table, i, i);
                TblSetItemStyle(table, i, 0, customTableItem);
                TblSetItemStyle(table, i, 1, checkboxTableItem);
                TblSetItemInt(table, i, 0, 1);
                TblSetRowUsable(table, i, true);
                TblMarkRowInvalid(table, i);

                TblDrawTable(table); 
            }
            
            // Draw the pin output tables - table D
                        
            table = FrmGetObjectPtr(form, FrmGetObjectIndex (form, portDTable));
            
            TblEraseTable(table);
            TblSetColumnUsable (table, 0, true);
            TblSetColumnUsable (table, 1, true);    
            
            TblSetCustomDrawProcedure (table, 0, customDrawPinD);
   
            for (i = 0; i < 4; i++)
            {
                TblSetRowID(table, i, i);
                TblSetItemStyle(table, i, 0, customTableItem);
                TblSetItemStyle(table, i, 1, checkboxTableItem);
                TblSetItemInt(table, i, 1, 1);
                TblSetRowUsable(table, i, true);
                TblMarkRowInvalid(table, i);

                TblDrawTable(table); 
            }
            
            FrmDrawForm(FrmGetActiveForm());
            gFormOpened = true;
	    handled = true;

	break;
			
	case menuEvent:		
	
            if (event->data.menu.itemID == aboutPalmLCD)
            {  
                FrmAlert(AboutAlert);

                handled = true;
            }
	
        break;

        case tblSelectEvent:


        
        break;
        
    }

    return(handled);
}


static Boolean ApplicationHandleEvent(EventPtr event)
{
    FormPtr	frm;
    Int		formId;
    Boolean	handled = false;

    if (event->eType == frmLoadEvent)
    {
        // Load the form resource specified in the event then activate the form.
	formId = event->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	// Set the event handler for the form.  The handler of the currently 
	// active form is called by FrmDispatchEvent each time it receives an event.
	switch (formId)
	{
	    case PalmLCDForm:
                
                FrmSetEventHandler(frm, MyFormHandleEvent);
	    
            break;
	}
	
        handled = true;
    }
	
    return handled;
}


static long TimeUntilNextRead(void)
{
    Long timeRemaining;

    if (!gFormOpened)
    {
        return evtWaitForever;
    }
    else 
    {
        timeRemaining = (TimGetTicks() + sysTicksPerSecond/50) - TimGetTicks();
    
        if (timeRemaining < 0)
            timeRemaining = 0;
        
        return timeRemaining;
    }
}


static void EventLoop(void)
{
    EventType	event;
    Word        error;
	
    do
    {
        EvtGetEvent(&event, TimeUntilNextRead());
	    if (! SysHandleEvent(&event))
		if (! MenuHandleEvent(0, &event, &error))
	            if (! ApplicationHandleEvent(&event))
		        FrmDispatchEvent(&event);
    }
    while (event.eType != appStopEvent);
}


UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    Err err = 0;		

    //gUpdateTimer = sysTicksPerSecond;
	
    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
	if ((err = StartApplication()) == 0)
	{
	    EventLoop();
		
            StopApplication();
	}
    }

    return err;
}
