/*
    PalmLCD - Accepts draw commands via serial port and acts as a slave

    Steve Manley (smanley@nyx.net)

    Feb 25th 2000 - Added Basic Functionality - ALL YOU BASE BELONG TO US	


    // Todos
    //
    // Get integer from checkboxes
    // Write integer to checkboxes
    // Send a ADC channel request
    // Read ADC value and write to screen
    // Read integer (port) value
    // Write integer port value
    //
    // Send S to start command, Z to send/reset/echo
    // 
    // SADC0Z - read adc
    // SWR[byte1][byte2]Z - adc value (merge)
    // SPB[byte]Z - write port b
    // SPD[byte]Z - recieve port D setting
*/

#include <PalmOS.h>
#include <PalmCompatibility.h>				
#include <System/SerialMgrOld.h>
#include <FloatMgr.h>

#include "palmlcdrsc.h"

// Application globals
UInt16   gSerialLib;
int      gSerialPortOpen;
short    gScreenBuffer[160];
char     gSerialBuffer[400];

char     gCommand[10];
char     gText[80];

char     gSerialCommand[10];

int      gCommandStarted;
int      gXpoint;
int      gYpoint;

ULong       gLastSuccessfulReception = 0;       
ULong       gLastTimeDisplay = 0;
ULong       gFormOpened = false;
ULong       gUpdateTimer;

unsigned char g_portB = 0;
unsigned char g_portD = 0;

int adcChannel = 0;
int adcValue = 0;

int g_adcType = 0;

int g_graphingEnabled = 1;

void drawScreen()
{
}


void setPoint(int x, int y, int value)
{
    int i,j;
    for (i=0; i<160; i++)
    {
            WinDrawLine(i,i,i,i);
    }
}


void handleSerial()
{

}

int gCommandStarted = 0;
int count = 0;
int numberGraphs = 0;

void eraseGraph()
{
    RectangleType dim;
    
    dim.topLeft.x = 20;
    dim.topLeft.y = 30;
    dim.extent.x  = 140;
    dim.extent.y  = 80;

    WinEraseRectangle(&dim, 0);

    /* draw axis, x & y */
    WinDrawLine(15,105,150,105);
    WinDrawLine(20,35,20,110);

    /* draw intermediate markers */
    WinDrawLine(15,80,20,80);
    WinDrawLine(15,55,20,55);               

    numberGraphs = 0;
}
            

/*
    Handle recieving commands from the serial port

*/



void handleSerialReceive(void)
{
    int     i;
    double  calcValue;
    double  maximum;
    int     scalefactor;
    int     input;
    char    data[80];
    char    temp[6];
    
    Err     err;
    ULong   numberBytesPending;
    ULong   numberBytesRead;
    Boolean handled;

    FormPtr form;
    TablePtr table;
    FieldPtr field;
    MemHandle textHandle;
    char* txt;

    char command;
    int index = 0;
    
    RectangleType dim;
    
    data[0] = '\0';
    
    SerReceiveFlush(gSerialLib, 1);
    
    form = FrmGetActiveForm();
     
    //SerSend(gSerialLib, "Handled:", 8, &err);
    
    do
    {
        SerReceiveWait(gSerialLib, sizeof(data) - 1, sysTicksPerSecond/50);
        SerReceiveCheck(gSerialLib, &numberBytesPending);

        if (numberBytesPending > 0) 
        {
            //numberBytesRead = SerReceive(gSerialLib, data, numberBytesPending, 0, &err);
            SerReceive(gSerialLib, data, numberBytesPending, 0, &err);

            //SerSend(gSerialLib, data,  numberBytesPending, &err);
            //SerSend(gSerialLib, "-Read\n\r", 7, &err);
                  
            while (numberBytesPending)
            {
                command = data[index];
                numberBytesPending--;
                index++;
            
            if ( (command == 'Z') && (gCommandStarted) )
            {
                gCommandStarted = 0;
                count = 0;
            
                //SerSend(gSerialLib, gSerialCommand,  StrLen(gSerialCommand), &err);

                if (g_graphingEnabled)
                {
                    numberGraphs++;
              
                    input = StrAToI(gSerialCommand);
                
                    maximum = 1023.0;
                
                    calcValue = (input / maximum) * 50.0;

                    scalefactor = calcValue;

                    if (scalefactor > 50)
                        scalefactor = 50;

                    dim.topLeft.x = 25 + (5 * numberGraphs);
                    dim.topLeft.y = 105 - scalefactor;
                    dim.extent.x  = 4;
                    dim.extent.y  = scalefactor;

                    WinDrawRectangle(&dim, 0);

                    if (numberGraphs > 22)
                    {
                        eraseGraph();
                        numberGraphs = 0;
                    }
                }
            }
            else if (gCommandStarted)
            {
                gSerialCommand[count] = command;
                count++;
                gSerialCommand[count] = '\0';
            }
            
            if (command == 'S')
            {
                gCommandStarted = 1;
                count = 0;
            }
            
            //StrCat(gSerialBuffer, data);
            //handleCommandRequest();

            SerReceiveFlush(gSerialLib, 1);
            SerClearErr(gSerialLib);
        }

        index = 0;

        }
    }
    while (!EvtSysEventAvail(false)); // Avoid blocking indefinately

}

static Err StartApplication(void)
{
    Err err;
    static SerSettingsType serialSettings;
    int i;

    FrmGotoForm(PalmLCDForm);

    if (SysLibFind ("Serial Library", &gSerialLib)) 
    {
        FrmAlert (SerialAlert);
    }

    err = SerOpen(gSerialLib, 0, 9600);

    if (err != 0) 
    {
        if (err == serErrAlreadyOpen) 
        {
            SerClose(gSerialLib);
            ErrNonFatalDisplay("Serial library is busy");
        } 
        else
        {  
            ErrNonFatalDisplay("Can't open serial library");
        }
 
        gSerialPortOpen = false;
 
        return true;
    }
    
    gSerialPortOpen = true;
    gCommandStarted = false;
    gSerialBuffer[0] = '\0';
    gText[0] = '\0';
    gXpoint = -1;
    gYpoint = -1;

    // Turn off hardware handshaking here
    serialSettings.baudRate = 9600;
    serialSettings.flags = 0;
    serialSettings.ctsTimeout = 0;      

    serialSettings.flags = serSettingsFlagStopBits1 | serSettingsFlagBitsPerChar8;

    err = SerSetSettings(gSerialLib, &serialSettings);

    if (err)
    {
        SerClearErr(gSerialLib);
        ErrDisplay("Error setting serial port");
    }

    SerSend(gSerialLib, "Start", 5, &err);

    /*

    err = SerSetReceiveBuffer(gSerialLib, gSerialBuffer, sizeof(gSerialBuffer));

    if (err != 0)
    {
        ErrNonFatalDisplay("Can't set serial buffer!");
    }
    */

    /* Set the screen to completely off */
    for (i=0; i<160; i++)
    {
        gScreenBuffer[i] = 0;
    }

   // ErrNonFatalDisplay("Done init..");

    return 0;
}


static void StopApplication(void)
{
    if (gSerialPortOpen) 
    {
        // SerSetReceiveBuffer(gSerialLib, NULL, 0);
        SerClose(gSerialLib);
    }
}


static Boolean MyFormHandleEvent(EventPtr event)
{
    int i;
    char temp[6];
    
    Boolean handled;
    Err err;

    ListPtr adcList;
    ControlPtr adcCtl;
    FormPtr form;
    
    TablePtr table;
    
    FieldPtr field;
    MemHandle textHandle;
    char* txt;
    
    RectangleType dim;
    
    handled = false;

    form = FrmGetActiveForm();
    
    switch (event->eType)
    {
        case nilEvent:

            handled = true;
            handleSerialReceive();

        break;
        
        case ctlSelectEvent:  // A control button was pressed and released.
   	    
            switch (event->data.ctlEnter.controlID)
            {
                case adcButton:

                    g_graphingEnabled = 0;

                    handled = true;
                    
                break;

                case portTableWrite:

                    g_graphingEnabled = 1;
                    
                    handled = true;
                    
                break;

                case portTableRead:

                    eraseGraph();
                
                    handled = true;

                break;                
            }
        
	
        break;

  	case frmOpenEvent:	

            FrmDrawForm(FrmGetActiveForm());

            eraseGraph();

            gFormOpened = true;
	    handled = true;

	break;
			
	case menuEvent:		
	
            if (event->data.menu.itemID == aboutPalmLCD)
            {  
                FrmAlert(AboutAlert);

                handled = true;
            }
	
        break;

        case tblSelectEvent:


        
        break;
        
    }

    return(handled);
}


static Boolean ApplicationHandleEvent(EventPtr event)
{
    FormPtr	frm;
    Int		formId;
    Boolean	handled = false;

    if (event->eType == frmLoadEvent)
    {
        // Load the form resource specified in the event then activate the form.
	formId = event->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	// Set the event handler for the form.  The handler of the currently 
	// active form is called by FrmDispatchEvent each time it receives an event.
	switch (formId)
	{
	    case PalmLCDForm:
                
                FrmSetEventHandler(frm, MyFormHandleEvent);
	    
            break;
	}
	
        handled = true;
    }
	
    return handled;
}


static long TimeUntilNextRead(void)
{
    Long timeRemaining;

    if (!gFormOpened)
    {
        return evtWaitForever;
    }
    else 
    {
        timeRemaining = (TimGetTicks() + sysTicksPerSecond/50) - TimGetTicks();
    
        if (timeRemaining < 0)
            timeRemaining = 0;
        
        return timeRemaining;
    }
}


static void EventLoop(void)
{
    EventType	event;
    Word        error;
	
    do
    {
        EvtGetEvent(&event, TimeUntilNextRead());
	    if (! SysHandleEvent(&event))
		if (! MenuHandleEvent(0, &event, &error))
	            if (! ApplicationHandleEvent(&event))
		        FrmDispatchEvent(&event);
    }
    while (event.eType != appStopEvent);
}


UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    Err err = 0;		

    //gUpdateTimer = sysTicksPerSecond;
	
    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
	if ((err = StartApplication()) == 0)
	{
	    EventLoop();
		
            StopApplication();
	}
    }

    return err;
}
