#include <PalmOS.h>
#include <PalmCompatibility.h>				
#include <System/SerialMgrOld.h>

#include "palmlcdrsc.h"

// Application globals
UInt16   gSerialLib;
int      gSerialPortOpen;
short    gScreenBuffer[160];
char     gSerialBuffer[400];

char     gCommand[10];
char     gText[80];

char     gSerialCommand[10];

int      gCommandStarted;
int      gXpoint;
int      gYpoint;

ULong       gLastSuccessfulReception = 0;       
ULong       gLastTimeDisplay = 0;
ULong       gFormOpened = false;
ULong       gUpdateTimer;

unsigned char g_portB = 0;
unsigned char g_portD = 0;

int adcChannel = 0;
int adcValue = 0;

int g_adcType = 0;


int g_samplingEnabled = 1;


void customDrawTime(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds);
void customDrawIndex(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds);
void customDrawSample(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds);
void listDatabase(void);

/* database reference */

#define DB_TYPE 'Data'
#define DB_APPID 'LCD7'
#define DB_NAME "SampleDB"

DmOpenRef   g_sampleDB;
int         g_numRecords;
UInt16      g_currentRecord;

typedef struct
{
    int sampleValue;
    char date[32];
}
sample_t;

void drawScreen()
{
}


void setPoint(int x, int y, int value)
{
    int i,j;
    for (i=0; i<160; i++)
    {
            WinDrawLine(i,i,i,i);
    }
}


void handleSerial()
{

}


void listDatabase()
{
    int i;
    UInt16 rec_num;
    UInt16 numberRecords;
    FormPtr form;
    TablePtr table;
    MemHandle rec_handle;
    
    sample_t* sample;
   
    form = FrmGetActiveForm();
    table = FrmGetObjectPtr(form, FrmGetObjectIndex (form, sampleTable));

    TblEraseTable(table);
    TblSetColumnUsable (table, 0, true);
    TblSetColumnUsable (table, 1, true);    
    TblSetColumnUsable (table, 2, true);    
            
    TblSetCustomDrawProcedure (table, 0, customDrawIndex);
    TblSetCustomDrawProcedure (table, 1, customDrawSample);
    TblSetCustomDrawProcedure (table, 2, customDrawTime);
    
    numberRecords = DmNumRecords(g_sampleDB);

    if (numberRecords > 10)
        numberRecords = 10;

    for (i = 0; i < numberRecords; i++) 
    {
        rec_handle = DmQueryRecord (g_sampleDB, i);
        
        if (!rec_handle)
            continue;

        sample = MemHandleLock (rec_handle);


        TblSetRowID(table, i, i);
        TblSetItemStyle(table, i, 0, customTableItem);
        TblSetItemStyle(table, i, 1, customTableItem);
        TblSetItemStyle(table, i, 2, customTableItem);
        TblSetRowUsable(table, i, true);
        TblMarkRowInvalid(table, i);
        
        MemHandleUnlock (rec_handle);

        TblSetRowUsable (table, i, true);
        TblSetRowSelectable (table, i, true);
        TblMarkRowInvalid (table, i);
   }
  
    // Hide the unused rows
    for (i = numberRecords; i < 10; i++)
    {            
        TblSetRowUsable(table, i, false);
    }
 
    TblDrawTable (table);    
}

void deleteRecords()
{
    int         i;
    Err         error;
    int         num_records;

    num_records = DmNumRecords(g_sampleDB);

    
    for (i = 1; i <= num_records; i++) 
    {
        error = DmRemoveRecord(g_sampleDB, (num_records - i));
    }

    
    // Update the table to reflect changes
    listDatabase();
}
     

void storeSample(int sample)
{
    UInt16 er;
    UInt16 currentRecord;
    FormPtr form;
    MemHandle rec_handle;
    MemHandle mem_handle;
    
    sample_t* new_sample;
    sample_t* tempSample;

    SystemPreferencesType sysPrefs;
    DateTimeType    date;
    Char            buffer[32];
    Char            time[32];
    char*           txt;
    MemHandle       txt_handle;
    
    txt_handle = MemHandleNew (32 + 1);
    txt = MemHandleLock (txt_handle);
    
    PrefGetPreferences (&sysPrefs);
    TimSecondsToDateTime(TimGetSeconds(), &date);   
    DateToAscii(date.month, date.day, date.year, sysPrefs.longDateFormat, txt);
    TimeToAscii(date.hour, date.minute, sysPrefs.timeFormat, time);
    StrCat(txt, " ");
    StrCat(txt, time);
    
    if (g_sampleDB)
    {
        if ( DmNumRecords(g_sampleDB) >= 10 )
        {
            MemHandleUnlock(txt_handle);            
            return;
        }
            
        currentRecord = DmNumRecords(g_sampleDB) + 1;
        rec_handle = DmNewRecord (g_sampleDB, &currentRecord, sizeof (sample_t));

        tempSample = MemPtrNew (sizeof (sample_t));
        MemSet (tempSample, sizeof (sample_t), 0);
        new_sample = MemHandleLock (rec_handle);

        tempSample->sampleValue = sample;
        StrCopy(tempSample->date, txt);

        DmWrite (new_sample, 0, tempSample, sizeof (sample_t));

        MemHandleUnlock (rec_handle);
        MemPtrFree (tempSample);
        DmReleaseRecord (g_sampleDB, currentRecord, true);
    }

    MemHandleUnlock(txt_handle);
}
/*
    Handle recieving commands from the serial port

*/

int gCommandStarted = 0;
int count = 0;

void handleSerialReceive(void)
{
    int     i;
    char    data[80];
    char    temp[6];
    Err     err;
    ULong   numberBytesPending;
    ULong   numberBytesRead;
    Boolean handled;

    FormPtr form;
    TablePtr table;
    FieldPtr field;
    MemHandle textHandle;
    char* txt;
   
    char command;
    int index = 0;
    
    data[0] = '\0';
    
    SerReceiveFlush(gSerialLib, 1);
    
    form = FrmGetActiveForm();
     
    //SerSend(gSerialLib, "Handled:", 8, &err);
    
    do
    {
        SerReceiveWait(gSerialLib, sizeof(data) - 1, sysTicksPerSecond/50);
        SerReceiveCheck(gSerialLib, &numberBytesPending);

        if (numberBytesPending > 0) 
        {
            //numberBytesRead = SerReceive(gSerialLib, data, numberBytesPending, 0, &err);
            SerReceive(gSerialLib, data, numberBytesPending, 0, &err);

            //SerSend(gSerialLib, data,  numberBytesPending, &err);
            //SerSend(gSerialLib, "-Read\n\r", 7, &err);
  
            while (numberBytesPending)
            {
                command = data[index];
                numberBytesPending--;
                index++;
                
            
            if ( (command == 'Z') && (gCommandStarted) )
            {
                gCommandStarted = 0;
                count = 0;
                
                if (g_samplingEnabled)
                {
                    storeSample(StrAToI(gSerialCommand)); 
                    listDatabase();
                }
                
                SerSend(gSerialLib, gSerialCommand,  StrLen(gSerialCommand), &err);

            }
            
            if (command == 'S')
            {
                gCommandStarted = 1;
                count = 0;
            }
            else if (gCommandStarted)
            {
                gSerialCommand[count] = command;
                count++;
                gSerialCommand[count] = '\0';
            }                  
    
            //StrCat(gSerialBuffer, data);
            
            //handleCommandRequest();
            SerReceiveFlush(gSerialLib, 1);
            SerClearErr(gSerialLib);
        }

        index = 0;

        }
    }
    while (!EvtSysEventAvail(false)); // Avoid blocking indefinately

}

static Err StartApplication(void)
{
    Err err;
    static SerSettingsType serialSettings;
    int i;

    FrmGotoForm(PalmLCDForm);

    g_sampleDB = DmOpenDatabaseByTypeCreator (DB_TYPE, DB_APPID, dmModeReadWrite);
        
    if (!g_sampleDB) 
    {
        err = DmCreateDatabase (0, DB_NAME, DB_APPID, DB_TYPE, false);
    
        if (!err) 
        {
            g_sampleDB = DmOpenDatabaseByTypeCreator (DB_TYPE, DB_APPID, dmModeReadWrite);
        }
    }  

    
    if (SysLibFind ("Serial Library", &gSerialLib)) 
    {
        FrmAlert (SerialAlert);
    }

    err = SerOpen(gSerialLib, 0, 9600);

    if (err != 0) 
    {
        if (err == serErrAlreadyOpen) 
        {
            SerClose(gSerialLib);
            ErrNonFatalDisplay("Serial library is busy");
        } 
        else
        {  
            ErrNonFatalDisplay("Can't open serial library");
        }
 
        gSerialPortOpen = false;
 
        return true;
    }
    
    gSerialPortOpen = true;
    gCommandStarted = false;
    gSerialBuffer[0] = '\0';
    gText[0] = '\0';
    gXpoint = -1;
    gYpoint = -1;

    // Turn off hardware handshaking here
    serialSettings.baudRate = 9600;
    serialSettings.flags = 0;
    serialSettings.ctsTimeout = 0;      

    serialSettings.flags = serSettingsFlagStopBits1 | serSettingsFlagBitsPerChar8;

    err = SerSetSettings(gSerialLib, &serialSettings);

    if (err)
    {
        SerClearErr(gSerialLib);
        ErrDisplay("Error setting serial port");
    }

    //SerSend(gSerialLib, "Start", 5, &err);

    /*

    err = SerSetReceiveBuffer(gSerialLib, gSerialBuffer, sizeof(gSerialBuffer));

    if (err != 0)
    {
        ErrNonFatalDisplay("Can't set serial buffer!");
    }
    */

    /* Set the screen to completely off */
    for (i=0; i<160; i++)
    {
        gScreenBuffer[i] = 0;
    }

   // ErrNonFatalDisplay("Done init..");

    return 0;
}


static void StopApplication(void)
{
    if (gSerialPortOpen) 
    {
        // SerSetReceiveBuffer(gSerialLib, NULL, 0);
        SerClose(gSerialLib);
    }
}

void customDrawIndex(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds)
{
    char *txt;
    char buffer[8];
    MemHandle mem_handle;
    FontID  old_font;

    mem_handle = MemHandleNew (16 + 1);
    txt = MemHandleLock (mem_handle);

    StrIToA(buffer, row);
    StrCopy(txt, "Sample");    
    txt[6] = buffer[0];
    txt[7] = '\0';

    old_font = FntSetFont (0);
    WinDrawChars (txt, StrLen(txt), bounds->topLeft.x, bounds->topLeft.y);
    FntSetFont (old_font);

    MemHandleUnlock (mem_handle);      
}


void customDrawTime(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds)
{
    char *txt;
    char buffer[32];
    sample_t *sample;
    UInt16 rec_num = 0;
    UInt16 number_records;
    MemHandle mem_handle, rec_handle;

    number_records = DmNumRecords(g_sampleDB);

    rec_num = TblGetRowID (tbl, row);
//    rec_handle = DmQueryRecord (g_sampleDB, (number_records - rec_num - 1));
    rec_handle = DmQueryRecord (g_sampleDB, rec_num );

    sample = MemHandleLock (rec_handle);
    mem_handle = MemHandleNew (32 + 1);
    txt = MemHandleLock (mem_handle);
    
    StrCopy(txt, sample->date);
    
    WinDrawChars (txt, StrLen(txt), bounds->topLeft.x, bounds->topLeft.y);

    MemHandleUnlock (rec_handle);
    MemHandleUnlock (mem_handle);
/*
    char *txt;
    MemHandle mem_handle;
    FontID  old_font;

    SystemPreferencesType sysPrefs;

    DateTimeType    date;
    Char            buffer[32];
    Char            time[32];
    
    
    mem_handle = MemHandleNew (32 + 1);
    txt = MemHandleLock (mem_handle);

    //StrCopy(txt, "23 Jun 03 04:00:00\0");    
    PrefGetPreferences (&sysPrefs);
    TimSecondsToDateTime(TimGetSeconds(), &date);   
    DateToAscii(date.month, date.day, date.year, sysPrefs.longDateFormat, txt);
    TimeToAscii(date.hour, date.minute, sysPrefs.timeFormat, time);
    
    StrCat(txt, " ");
    StrCat(txt, time);
    
    
    old_font = FntSetFont (0);
    WinDrawChars (txt, StrLen(txt), bounds->topLeft.x, bounds->topLeft.y);
    FntSetFont (old_font);

    MemHandleUnlock (mem_handle);      
*/    
}

void customDrawSample(MemPtr tbl, UInt16 row, UInt16 column, RectanglePtr bounds)
{
    char *txt;
    char buffer[16];
    sample_t *sample;
    UInt16 rec_num = 0;
    UInt16 number_records;
    MemHandle mem_handle, rec_handle;

    number_records = DmNumRecords(g_sampleDB);

    rec_num = TblGetRowID (tbl, row);
    //rec_handle = DmQueryRecord (g_sampleDB, (number_records - rec_num - 1));
    rec_handle = DmQueryRecord (g_sampleDB, rec_num);

    sample = MemHandleLock (rec_handle);
    mem_handle = MemHandleNew (8 + 1);
    txt = MemHandleLock (mem_handle);
    
    buffer[0] = '\0';
    buffer[1] = '\0';
    buffer[2] = '\0';
    buffer[3] = '\0';
    buffer[4] = '\0';
    
    StrIToA(buffer, sample->sampleValue);
    StrCopy(txt, buffer);
    
    WinDrawChars (buffer, StrLen(buffer), bounds->topLeft.x, bounds->topLeft.y);

    MemHandleUnlock (rec_handle);
    MemHandleUnlock (mem_handle);
/*
    char *txt;
    char buffer[5];
    MemHandle mem_handle;
    FontID  old_font;

    mem_handle = MemHandleNew (16 + 1);
    txt = MemHandleLock (mem_handle);

    StrIToA(buffer, row);
    StrCopy(txt, "000");    
    txt[3] = buffer[0];
    txt[4] = '\0';

    old_font = FntSetFont (0);
    WinDrawChars (txt, StrLen(txt), bounds->topLeft.x, bounds->topLeft.y);
    FntSetFont (old_font);

    MemHandleUnlock (mem_handle);      
*/
}


static Boolean MyFormHandleEvent(EventPtr event)
{
    int i;
    char temp[6];
    
    Boolean handled;
    Err err;

    ListPtr adcList;
    ControlPtr adcCtl;
    FormPtr form;
    
    TablePtr table;
    
    FieldPtr field;
    MemHandle textHandle;
    char* txt;
    
    handled = false;

    form = FrmGetActiveForm();
    
    switch (event->eType)
    {
        case nilEvent:

            handled = true;
            handleSerialReceive();

        break;
        
        case ctlSelectEvent:  // A control button was pressed and released.
   	    
            switch (event->data.ctlEnter.controlID)
            {
                // Sample new data
                case adcButton:

                    if (g_samplingEnabled)
                        g_samplingEnabled = 0;
                    else
                        g_samplingEnabled = 1;
                    
                    listDatabase();
                    
                    handled = true;
                    
                break;

                // Load existing data
                case portTableWrite:

                    
                    handled = true;
                    
                break;


                // Clear the table
                case portTableRead:

                    deleteRecords();
                    listDatabase();
                    
                    handled = true;

                break;                
            }
        
	
        break;

  	case frmOpenEvent:	
            
            // Draw the sample tables
/*
            table = FrmGetObjectPtr(form, FrmGetObjectIndex (form, sampleTable));
            
            TblEraseTable(table);
            TblSetColumnUsable (table, 0, true);
            TblSetColumnUsable (table, 1, true);    
            TblSetColumnUsable (table, 2, true);    
            
            TblSetCustomDrawProcedure (table, 0, customDrawIndex);
            TblSetCustomDrawProcedure (table, 1, customDrawSample);
            TblSetCustomDrawProcedure (table, 2, customDrawTime);
   
            for (i = 0; i < 10; i++)
            {
                TblSetRowID(table, i, i);
                TblSetItemStyle(table, i, 0, customTableItem);
                TblSetItemStyle(table, i, 1, customTableItem);
                TblSetItemStyle(table, i, 2, customTableItem);
                TblSetRowUsable(table, i, true);
                TblMarkRowInvalid(table, i);
            }
            
            TblDrawTable(table); 
 */           

            listDatabase();
        
            FrmDrawForm(FrmGetActiveForm());
            gFormOpened = true;
	    handled = true;

	break;
			
	case menuEvent:		
	
            if (event->data.menu.itemID == aboutPalmLCD)
            {  
                FrmAlert(AboutAlert);

                handled = true;
            }
	
        break;

        case tblSelectEvent:


        
        break;
        
    }

    return(handled);
}


static Boolean ApplicationHandleEvent(EventPtr event)
{
    FormPtr	frm;
    Int		formId;
    Boolean	handled = false;

    if (event->eType == frmLoadEvent)
    {
        // Load the form resource specified in the event then activate the form.
	formId = event->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	// Set the event handler for the form.  The handler of the currently 
	// active form is called by FrmDispatchEvent each time it receives an event.
	switch (formId)
	{
	    case PalmLCDForm:
                
                FrmSetEventHandler(frm, MyFormHandleEvent);
	    
            break;
	}
	
        handled = true;
    }
	
    return handled;
}


static long TimeUntilNextRead(void)
{
    Long timeRemaining;

    if (!gFormOpened)
    {
        return evtWaitForever;
    }
    else 
    {
        timeRemaining = (TimGetTicks() + sysTicksPerSecond/50) - TimGetTicks();
    
        if (timeRemaining < 0)
            timeRemaining = 0;
        
        return timeRemaining;
    }
}


static void EventLoop(void)
{
    EventType	event;
    Word        error;
	
    do
    {
        EvtGetEvent(&event, TimeUntilNextRead());
	    if (! SysHandleEvent(&event))
		if (! MenuHandleEvent(0, &event, &error))
	            if (! ApplicationHandleEvent(&event))
		        FrmDispatchEvent(&event);
    }
    while (event.eType != appStopEvent);
}


UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    Err err = 0;		

    //gUpdateTimer = sysTicksPerSecond;
	
    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
	if ((err = StartApplication()) == 0)
	{
	    EventLoop();
		
            StopApplication();
	}
    }

    return err;
}
